@@ -28,7 +28,7 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
   fosr.obj <- obj
 
   ### NULLify global values called in ggplot
-  value = subj = covariate = UB = LB = residual = depth.rank = coef = grid = NULL
+  value = subj = covariate = UB = LB = residual = depth.rank = coef = grid = x = y = NULL
 
   ################################
   ## code for processing tabs
@@ -43,19 +43,36 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
   covar.list[1] = "None"
   covarInputValues = 1:length(covar.list)
   names(covarInputValues) = covar.list
-
+  spaghetti.help = "Observed response data, colored according to the covariate selected below."
+  spaghetti.call = eval(call("selectInput", inputId = "CovarChoice", label = ("Select Covariate"), choices = covarInputValues, selected = 1))
+  lasagna.help = "Observed response data, each row is a subject. When a covariate is selected, the rows are ordered and widths 
+                are assigned by value of that covariate. Right plot displays distribution of covariate selected in 
+                correspondence with ordering of the rows at right."
+  lasagna.call = eval(call("selectInput", inputId = "CovarChoice2", label = ("Select Covariate"), choices = covarInputValues, selected = 1))
+  
+  
   ## Tab 2: fitted values
   pred.list = names(attributes(terms(fosr.obj$terms))$dataClasses)[-1]
-  calls <- vector("list", length(pred.list))
+  fitted.help = "Fitted response curve for a subject with covariate values specified below."
+  fitted.call <- vector("list", length(pred.list))
   for(i in 1:length(pred.list)){
-    calls[[i]] =  eval(createInputCall(pred.list[i], get(pred.list[i], fosr.obj$data) ))
+    fitted.call[[i]] =  eval(createInputCall(pred.list[i], get(pred.list[i], fosr.obj$data) ))
   }
 
   ## Tab 3: coefficient functions
   coef.list = colnames(model.matrix(fosr.obj$terms, fosr.obj$data[1,]))
   coefInputValues = 1:p
   names(coefInputValues) = coef.list
-
+  coef.help = "Coefficient function and confidence bounds for the predictor selected below."
+  coef.call = eval(call("selectInput", inputId = "CoefChoice", label = ("Select Predictor"), choices = coefInputValues, selected = 1))
+  
+  ## Tab 4: plot of residual curves
+  residuals.help = "If 'Show Outliers' is selected, the median and outlying curves are shown in blue and red respectively. If 'Rainbowize' 
+                    is selected, curves are ordered by band depth with most outlying curves shown in red and 
+                    curves closest to the median shown in violet."
+  residuals.call = eval(call("radioButtons","residOptions", label="Plot Options", 
+                             choices = list("None"=1, "Show Median and Outliers"=2,"Rainbowize by Depth"=3), selected=1))
+  
   #################################
   ## App
   #################################
@@ -66,59 +83,20 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
   ## UI
   #################################
 
-    ui = navbarPage(title = strong(style = "color: #ACD6FF; padding: 0px 0px 10px 10px; opacity: 0.95; ", "FoSR Plot"), windowTitle = "refund.shiny",
-                    collapsible = FALSE, id = "nav",
-                    inverse = TRUE, header = NULL,
+    ui = navbarPage(title = strong(style = "color: #ACD6FF; padding: 0px 0px 10px 10px; opacity: 0.95; ", "FoSR Plot"), 
+                    windowTitle = "refund.shiny", collapsible = FALSE, id = "nav", inverse = TRUE, header = NULL,
+                    ##### start tabs
                     tabPanel("Observed Data", icon = icon("stats", lib = "glyphicon"),
-                             column(3,
-                                    helpText("Observed response data, colored according to the covariate selected below."), hr(),
-                                    selectInput("CovarChoice", label = ("Select Covariate"), choices = covarInputValues, selected = 1),hr(),
-                                    downloadButton('downloadPDFObs', "Download Plot as PDF"), br(), br(),
-                                    downloadButton("downloadPlotObs", "Download Plot as Object", class = "plot-download")
-                                    ),
-                             column(9, h4("Observed Data"),
-                                    plotOutput('ObsDataPlot')
-                                    )
-                            ),
-                    tabPanel("Fitted Values", icon = icon("line-chart"),
-                             column(3,
-                                    helpText("Fitted response curve for a subject with covariate values specified below."), hr(),
-                                    eval(calls), hr(),
-                                    downloadButton('downloadPDFFittedVal', "Download Plot as PDF"), br(), br(),
-                                    downloadButton("downloadPlotFittedVal", "Download Plot as Object", class = "plot-download")
-                                   ),
-                             column(9, h4("Fitted Response Curve"),
-                                   plotOutput('FittedValPlot')
-                                   )
-                            ),
-                    tabPanel("Coefficient Functions", icon = icon("area-chart"),
-                             column(3,
-                                    helpText("Coefficient function and confidence bounds for the predictor selected below"), hr(),
-                                    selectInput("CoefChoice", label = ("Select Predictor"), choices = coefInputValues, selected = 1), hr(),
-                                    downloadButton('downloadPDFCoefFunc', "Download Plot as PDF"), br(), br(),
-                                    downloadButton("downloadPlotCoefFunc", "Download Plot as Object", class = "plot-download")
-                                    ),
-                             column(9, h4("Coefficient Function"),
-                                    plotOutput('CoefFunc')
-                                    )
-                            ),
-                    tabPanel("Residuals", icon = icon("medkit"),
-                             column(3,
-                                    helpText("Plot of residual curves."), hr(),
-                                    #checkboxInput("outliers", label="Show median and outliers"),
-                                    radioButtons("residOptions", label="Plot Options",
-                                                 choices = list("None"=1, "Show Median and Outliers"=2,"Rainbowize by Depth"=3),
-                                                 selected=1),
-                                    helpText("If 'Show Outliers' is selected, the median and outlying curves are shown
-                                             in blue and red respectively. If 'Rainbowize' is selected, curves are ordered by band depth
-                                             with most outlying curves shown in red and curves closest to the median shown in violet"), hr(),
-                                    downloadButton('downloadPDFresid', "Download Plot as PDF"), br(), br(),
-                                    downloadButton("downloadPlotresid", "Download Plot as Object", class = "plot-download")
-                                    ),
-                             column(9, h4("Residuals"),
-                                    plotOutput('resid')
-                                    )
-                            )
+                             tabsetPanel(
+                               tabPanelModuleUI("spaghetti", tabTitle = "Spaghetti Plot", calls = spaghetti.call,
+                                                helperText = spaghetti.help),
+                               tabPanelModuleUI("lasagna", tabTitle = "Lasagna Plot", calls = lasagna.call,
+                                                helperText = lasagna.help)
+                             ) ),
+                    tabPanelModuleUI("fitted", tabTitle = "Fitted Values", icon("line-chart"), calls = fitted.call,helperText = fitted.help ),
+                    tabPanelModuleUI("coef", tabTitle = "Coefficient Functions", icon("area-chart"), calls = coef.call, helperText = coef.help),
+                    tabPanelModuleUI("residuals", tabTitle = "Residuals", icon("medkit"), calls = residuals.call,helperText = residuals.help )
+                    ##### end tabs
                     ),
 
     #################################
@@ -131,7 +109,7 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
       ## Code for observed data tab
       #################################
 
-      plotInputObsData <- reactive({
+      plotInputSpagheti <- reactive({
         y.obs = fosr.obj$data[,names(attributes(terms(fosr.obj$terms))$dataClasses)[1]]
         colnames(y.obs) = grid
         y.obs.m = melt(y.obs)
@@ -155,11 +133,27 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
         }
       })
 
-      output$ObsDataPlot <- renderPlot(print(plotInputObsData()) )
-
-      output$downloadPDFObs <- savePDF("ObsData.pdf", plotInputObsData())
-      output$downloadPlotObs <- savePlot("ObsData.RData", plotInputObsData())
-
+      
+      plotInputLasagna = reactive({
+        y.obs.char = as.character(fosr.obj$terms[[2]]) ## gets character string which is name of outcome variable
+        
+        CovarChoice2 = as.numeric(input$CovarChoice2)
+        selected = covar.list[CovarChoice2]
+        if(selected == "None") {
+         covariate = NULL
+        } else {
+         covariate = selected
+        }
+        
+        df = makeLasagna(data = fosr.obj$data, outcome = y.obs.char, covariate = covariate)
+        plots = bakeLasagna(data = fosr.obj$data, data.long = df$data.long, covariate = covariate)
+        
+        grid.arrange(plots$lasagnaPlot, plots$densityPlot, ncol=2, nrow=1, widths=c(4, 1))
+      })
+      
+      callModule(tabPanelModule, "spaghetti", plotObject = plotInputSpagheti, plotName = "spaghetti")
+      callModule(tabPanelModule, "lasagna", plotObject = plotInputLasagna, plotName = "lasagna", is.grid = TRUE)
+      
       #################################
       ## Code for FittedValues Tab
       #################################
@@ -192,11 +186,8 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
           xlab(xlab) + ylab(ylab) + ylim(c(.9, 1.1) * range(fosr.obj$Yhat))
 
       })
-
-      output$FittedValPlot <- renderPlot(print(plotInputFittedVal()) )
-
-      output$downloadPDFFittedVal <- savePDF("FittedVal.pdf", plotInputFittedVal())
-      output$downloadPlotFittedVal <- savePlot("FittedVal.RData", plotInputFittedVal())
+      
+      callModule(tabPanelModule, "fitted", plotObject = plotInputFittedVal, plotName = "fitted")
 
       #################################
       ## Code for CoefFunc Tab
@@ -216,11 +207,9 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
 
       })
 
-      output$CoefFunc <- renderPlot(print(plotInputCoefFunc()) )
-
-      output$downloadPDFCoefFunc <- savePDF("CoefFunc.pdf", plotInputCoefFunc())
-      output$downloadPlotCoefFunc <- savePlot("CoefFunc.RData", plotInputCoefFunc())
-
+      
+      callModule(tabPanelModule, "coef", plotObject = plotInputCoefFunc, plotName = "coef")
+      
       #################################
       ## Code for Residual plot
       #################################
@@ -267,15 +256,10 @@ plot_shiny.fosr = function(obj, xlab = "", ylab="", title = "", ...) {
         residPlot  + xlab("") + ylab("")
       })
 
-
-      output$resid <- renderPlot(plotInputResid() )
-
-      output$downloadPDFresid <- savePDF("resid.pdf", plotInputResid())
-      output$downloadPlotresid <- savePlot("resid.RData", plotInputResid())
-
+       callModule(tabPanelModule, "residuals", plotObject = plotInputResid, plotName = "residuals")
 
       ## add subject number
-
+  
     } ## end server
   )
 }
